<?xml version="1.0"?>
<package format="3">
    <name>{{ id }}</name>
    <version>0.1.0</version>
    <description>{{ name }}</description>

  <!-- One maintainer tag required, multiple allowed, one person per tag -->
  <!-- Example:  -->
  <!-- <maintainer email="jane.doe@example.com">Jane Doe</maintainer> -->
  <maintainer email="todo@todo.todo">TODO</maintainer>

  <!-- One license tag required, multiple allowed, one license per tag -->
  <!-- Commonly used license strings: -->
  <!--   BSD, MIT, Boost Software License, GPLv2, GPLv3, LGPLv2.1, LGPLv3 -->
  <license>Apache-2.0</license>

  <!-- Url tags are optional, but multiple are allowed, one per tag -->
  <!-- Optional attribute type can be: website, bugtracker, or repository -->
  <!-- Example: -->
  <!-- <url type="website">http://wiki.ros.org/test</url> -->


  <!-- Author tags are optional, multiple are allowed, one per tag -->
  <!-- Authors do not have to be maintainers, but could be -->
  <!-- Example: -->
  <!-- <author email="jane.doe@example.com">Jane Doe</author> -->


  <!-- The *depend tags are used to specify dependencies -->
  <!-- Dependencies can be catkin packages or system dependencies -->
  <buildtool_depend>ament_cmake_auto</buildtool_depend>

  <depend>diagnostic_msgs</depend>
  <depend>kb_msgs</depend>
  <depend>rclcpp</depend>
  <depend>rclcpp_action</depend>
  <depend>rclcpp_lifecycle</depend>
  <depend>sample_skill_msgs</depend>

  <build_depend>diagnostic_updater</build_depend>
  <build_depend>lifecycle_msgs</build_depend>
  <build_depend>nlohmann-json-dev</build_depend>
  <build_depend>std_skills</build_depend>

  <exec_depend>diagnostic_updater</exec_depend>
  <exec_depend>launch</exec_depend>
  <exec_depend>launch_ros</exec_depend>
  <exec_depend>launch_pal</exec_depend>
  <exec_depend>lifecycle_msgs</exec_depend>
  <exec_depend>std_skills</exec_depend>

  <test_depend>ament_lint_auto</test_depend>
  <test_depend>ament_lint_common</test_depend>

  <export>
      <build_type>ament_cmake</build_type>

      <skill content-type="json">
          {
            "id": "{{ id }}",
            "description": "This is a basic example skill. Write here an actual description.",
            "interface": "action",
            "default_interface_path": "/{{ id }}/control",
            "datatype": "sample_skill_msgs/action/SkillControl",
            "functional_domains": [
                "other"
            ]
        }
      </skill>
  </export>
</package>
