# sample action message for the 'Greet' task. This is purely
# for illustration purpose! You would probably not implement
# a 'greet' task using this message.

# define some status constants, used for feedback
uint8 INACTIVE = 0
uint8 SELECTING_USER = 1
uint8 FACING_USER = 2
uint8 GREETING_USER = 3
uint8 SUCCESSFUL_USER_ACKNOWLEDGMENT = 4

string person_id

# optional custom greeting message
string custom_greetings
---
string error_msg
---
string status

