<?xml version="1.0"?>
<package format="3">
  <name>{{ id }}</name>
  <version>0.1.0</version>
  <description>{{ name }}</description>

  <!-- One maintainer tag required, multiple allowed, one person per tag -->
  <!-- Example:  -->
  <!-- <maintainer email="jane.doe@example.com">Jane Doe</maintainer> -->
  <maintainer email="todo@todo.todo">TODO</maintainer>

  <!-- One license tag required, multiple allowed, one license per tag -->
  <!-- Commonly used license strings: -->
  <!--   BSD, MIT, Boost Software License, GPLv2, GPLv3, LGPLv2.1, LGPLv3 -->
  <license>Apache-2.0</license>

  <!-- Url tags are optional, but multiple are allowed, one per tag -->
  <!-- Optional attribute type can be: website, bugtracker, or repository -->
  <!-- Example: -->
  <!-- <url type="website">http://wiki.ros.org/test</url> -->


  <!-- Author tags are optional, multiple are allowed, one per tag -->
  <!-- Authors do not have to be maintainers, but could be -->
  <!-- Example: -->
  <!-- <author email="jane.doe@example.com">Jane Doe</author> -->


  <!-- The *depend tags are used to specify dependencies -->
  <!-- Dependencies can be catkin packages or system dependencies -->
  <depend>rclpy</depend>
  <depend>lifecycle_msgs</depend>

  <exec_depend>action_msgs</exec_depend>
  <exec_depend>launch</exec_depend>
  <exec_depend>launch_ros</exec_depend>
  <exec_depend>launch_pal</exec_depend>

  <exec_depend>task_msgs</exec_depend>

  <test_depend>ament_copyright</test_depend>
  <test_depend>ament_flake8</test_depend>
  <test_depend>ament_pep257</test_depend>
  <test_depend>python3-pytest</test_depend>

  <export>
      <build_type>ament_python</build_type>

      <task content-type="json">
        {
          "name": "{{id}}",
            "description": "This task implements ...",
            "interface": "action",
            "default_interface_path": "/{{id}}/control",
            "datatype": "task_msgs/action/TaskControl",
        }
      </task>
  </export>
</package>
