# sample action message to control (start/stop) a generic task.
# This is for illustration purpose: you probably will want to
# eg pass custom data to configure your task, or return specific
# values: feel free to modify it as needed.

string task_data
---
string error_msg
string result
---
string status

