# Copyright (c) 2024 PAL Robotics S.L. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

cmake_minimum_required(VERSION 3.8)
project({{id}})

# Global flags
if(NOT CMAKE_CXX_STANDARD)
  set(CMAKE_CXX_STANDARD 17)
  set(CMAKE_CXX_STANDARD_REQUIRED ON)
endif()

# Dependencies
find_package(ament_cmake_auto REQUIRED)
ament_auto_find_build_dependencies()

# Targets
ament_auto_add_library(${PROJECT_NAME}_lib DIRECTORY src/{{id}})
ament_auto_add_executable(start_skill src/start_skill.cpp)

# Additional installed resources
ament_index_register_resource(pal_configuration.${PROJECT_NAME} CONTENT config/00-defaults.yml)

if(pal_module_cmake_FOUND)
  pal_register_modules(module/${PROJECT_NAME}_module.yaml)
#  uncomment the next line to make the application be launched automatically at startup
#  pal_register_modules_sets(module_set/${PROJECT_NAME}_module_set.yaml)
endif()

# Testing
if(BUILD_TESTING)
  ament_auto_find_test_dependencies()
  ament_lint_auto_find_test_dependencies()
endif()

ament_auto_package(INSTALL_TO_SHARE config launch module module_set)
